package MovementStrategy;

public abstract class AbstractStrategy {
    private IMoveableObject _moveableObject;
    private Status _state = Status.NotInit;
    protected int FieldWidth;
    protected int FieldHeight;
    public Status GetStatus() { return _state; }

    // Изменить статус, установить поля
    public void SetData(IMoveableObject moveableObject, int width, int height)
    {
        if (moveableObject == null)
        {
            _state = Status.NotInit;
            return;
        }
        _state = Status.InProgress;
        _moveableObject = moveableObject;
        FieldWidth = width;
        FieldHeight = height;
    }

    // сделать шаг
    public void MakeStep()
    {
        if (_state != Status.InProgress)
        {
            return;
        }
        if (IsTargetDestination())
        {
            _state = Status.Finish;
            return;
        }
        MoveToTarget();
    }

    // перемещения
    protected boolean MoveLeft() { return MoveTo(Direction.Left); }
    protected boolean MoveRight() { return MoveTo(Direction.Right); }
    protected boolean MoveUp() { return MoveTo(Direction.Up); }
    protected boolean MoveDown() { return MoveTo(Direction.Down); }

    // параметры
    protected ObjectParameters GetObjectParameters() { return _moveableObject.GetObjectPosition(); }
    // шаг
    protected int GetStep()
    {
        if (_state != Status.InProgress)
        {
            return 0;
        }
        return _moveableObject.GetStep();
    }
    // перемещение
    protected abstract void MoveToTarget();

    // достигнута ли цель
    protected abstract boolean IsTargetDestination();

    // попытка перемещения по направлению
    private boolean MoveTo(Direction directionType)
    {
        if (_state != Status.InProgress)
        {
            return false;
        }
        if (_moveableObject.CheckCanMove(directionType))
        {
            _moveableObject.MoveObject(directionType);
            return true;
        }
        return false;
    }
}
